import { event, hierarchy } from 'd3'

export const render = (svg, props) => {
  const { root, color = '#dedbef' } = props
  
  let links = svg.append('g').selectAll('path').data(root.links())
  let linksExit = links.exit().remove()
  let linksEnter = links.enter().append('path')
    .attr('fill', 'none')
    .attr('stroke', color)
  
  links = links.merge(linksEnter)
    .attr('d', d => 
      `M${d.source.y / 2},${d.source.x}
			 V${d.target.x}
			 H${d.target.y / 2}`
    )

  let circles = svg.append('g').selectAll('circle')
    .data(root.descendants(), d => d.id)
  let circlesExit = circles.exit().remove()
  let circlesEnter = circles.enter().append('circle')
  	.attr('r', 10)
  	.attr('fill', color)
  circles = circles.merge(circlesEnter)
    .attr('cx', (d) => d.y / 2)
    .attr('cy', (d) => d.x)
    .on('click', d => {
      if (event.metaKey) {
        console.log("removing subtree...", d)
        const index = d.parent.children.indexOf(d)
        // remove this subtree from parent
        d.parent.children.splice(index, 1)
        render(svg, { root, color: 'blue' })
      }
    	else if (event.shiftKey) {
        d.children = d.children || []
        d.children.push(hierarchy({
          id: Math.random(),
          name: "florps",
          type: "thingy"
        }))
        render(svg, { root, color: 'red' })
      }
    	else {
        console.log(d)
      }
    })
}